#ifndef ALTPARSER_H
#define ALTPARSER_H 

#include <iostream>


enum class Instruction : char
{
    PLUS,
    MINUS,
    LEFT,
    RIGHT,
    DOT,
    COMMA,
    LEFT_PARAN,
    RIGHT_PARAN
    INVALID
};


struct BfIterator
{
    std::istream& source;
    Instruction currentInst;

    inline BfIterator(std::istream& source) : source(source) {}
    BfIterator(const BfIterator& other) = delete;
    inline BfIterator(BfIterator&& other) :
        source(other.source), currentInst(other.currentInst) {}

    inline BfIterator& operator++(void)
    {
        while (!source.eof()) {
            int chr = source.get();
            switch (chr) {
                case '+':
                    currentInst = Instruction::PLUS;
                    return *this;
                case '-':
                    currentInst = Instruction::MINUS;
                    return *this;
                case '<':
                    currentInst = Instruction::LEFT;
                    return *this;
                case '>':
                    currentInst = Instruction::RIGHT;
                    return *this;
                case '.':
                    currentInst = Instruction::DOT;
                    return *this;
                case ',':
                    currentInst = Instruction::COMMA;
                    return *this;
                case '[':
                    currentInst = Instruction::LEFT_PARAN;
                    return *this;
                case ']':
                    currentInst = Instruction::RIGHT_PARAN;
                    return *this;
            }
        }
        currentInst = Instruction::INVALID;
        return *this;
    }

    inline Instruction operator*(void) const
    {
        return currentInst;
    }

    inline bool operator != (const BfIterator& other) {
        return other.source.eof() != source.eof();
    }
};


class BlockIterator
{
    BfIterator& it;
    // iterate, until special character

    bool isSpecialCharacter(void)
    {
        return *it == Instruction::LEFT_PARAN ||
               *it == Instruction::RIGHT_PARAN ||
               *it == Instruction::COMMA ||
               *it == Instruction::DOT;
    }
public:
    BlockIterator& operator++(void)
    {
        if (!isSpecialCharacter())
            it++;
        return *this;
    }

    Instruction operator *(void)
    {
        return *it;
    }
};


class BlockParser
{
    BfIterator& begin;
public:
    inline BlockIterator begin(void) { return begin; }
    inline BlockIterator& end(void) { return ; }
};


template<typename Visitor, typename Value>
struct LoopIterator
{
    BfIterator& it;
    Visitor& visitor;
    Value value;

    LoopIterator(const LoopIterator&) = delete;

    LoopIterator& operator++(void)
    {
        if (*it == Instruction::RIGHT_PARAN)
            return *this;
        if (*it == Instruction::LEFT_PARAN) {
            value = visitor(LoopParser(it))
        }
        else {
            value = visitor(BlockParser(it));
        }
        return *this;
    }


    Value& operator * (void)
    {
        return value;
    }

    // until ']'
    //
    // calls d.loop() and d.block()                                 
};


#endif /* ALTPARSER_H */
