#include "Parser.h"
#include "Optimizer.h"
#include <string>

using namespace zp;
using namespace std;


std::unique_ptr<BlockInstruction> InstructionBlock::accept(AstVisitor& v)
{
    return v.visitInstructionBlock(*this);
}


std::unique_ptr<BlockInstruction> IOBlock::accept(AstVisitor& v)
{
    return v.visitIOBlock(*this);
}


std::unique_ptr<BlockInstruction> UnionBlock::accept(AstVisitor& v)
{
    return v.visitUnionBlock(*this);
}


std::unique_ptr<BlockInstruction> Loop::accept(AstVisitor& v)
{
    return v.visitLoop(*this);
}


Parser::Parser(istream& in) :
    in{in}
{
}


unique_ptr<Block> Parser::parse(void)
{
    return parseBlock(false);
}


unique_ptr<Block> Parser::parseBlock(bool isLoop)
{
    unique_ptr<UnionBlock> wrapper = nullptr;
    unique_ptr<InstructionBlock> block = make_unique<InstructionBlock>();

    char chr;
    while (in >> chr) {
        switch(chr) {
            case '+':
                block->addInstruction(Instruction::PLUS);
                break;
            case '-':
                block->addInstruction(Instruction::MINUS);
                break;
            case '<':
                block->addInstruction(Instruction::LEFT);
                break;
            case '>':
                block->addInstruction(Instruction::RIGHT);
                break;
            case '.':
            case ',':
                if (wrapper == nullptr) {
                    wrapper = isLoop ? make_unique<Loop>() : make_unique<UnionBlock>();
                }
                if (!block->isEmpty()) {
                    wrapper->addBlock(move(block));
                    block = make_unique<InstructionBlock>();
                }
                wrapper->addBlock(make_unique<IOBlock>(chr == '.' ? Instruction::DOT : Instruction::COMMA));
                break;
            case '[':
                if (wrapper == nullptr) {
                    wrapper = isLoop ? make_unique<Loop>() : make_unique<UnionBlock>();
                }
                if (!block->isEmpty()) {
                    wrapper->addBlock(move(block));
                    block = make_unique<InstructionBlock>();
                }
                wrapper->addBlock(parseBlock(true));
                break;
            case ']':
                if (isLoop) {
                    if (wrapper == nullptr)
                        wrapper = make_unique<Loop>();
                    if (!block->isEmpty())
                        wrapper->addBlock(move(block));
                    return wrapper;
                }
                else
                    throw ParserException("encountered unexpected ']'");
            default:;
        }
    }
    if (isLoop)
        throw ParserException("number of '['s and ']'s not matching");
    else if (wrapper == nullptr)
        return block;
    else
    {
        if (!block->isEmpty())
            wrapper->addBlock(move(block));
        return wrapper;
    }
}
